Unit Misc;

Interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ShellApi;

procedure CaptureScreenRect( ARect: TRect; ABitmap: TBitmap );
function DiskTypeStr(Drive: Char): ShortString;
function DiskInDrive(Drive: Char): Boolean;
function ExecuteFile(const FileName: string): Boolean;
function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
function IsASCIIFile(Fn: String): Boolean;
procedure Sleep(ms: Integer);

Implementation


procedure Sleep(ms: Integer);
var Start: LongInt;
begin
  Start := GetTickCount;
  repeat
    Application.ProcessMessages;
  until (GetTickCount > Start + ms) or (GetTickCount < Start)
end;

function IsASCIIFile(Fn: String): Boolean;
var Buf: array[0..255] of Byte;
    F: File;
    BR, i: Integer;
const ControlChars = [1..31] - [10,13,26];
      //Alle ASCII tekens beneden spatie, m.u.v. #0, LineFeed, CarriageReturn, EOF
begin
  Result := False;
  AssignFile(F,Fn);
  Try
    Reset(F,1);
    BlockRead(F,Buf,SizeOf(Buf),BR);
    CloseFile(F);
    if BR < 1 then Exit;
    for i := 0 to BR - 1 do
    begin
      if Buf[i] in ControlChars then Exit;
    end;
    Result := true;
  Except
    {$I-}CloseFile(F); InOutRes := 0; {$I+}
  end;
end;



procedure CaptureScreenRect( ARect: TRect; ABitmap: TBitmap );
var
  ScreenDC: HDC;
begin
  with ABitmap, ARect do
  begin
    Width := Right - Left;
    Height := Bottom - Top;
    ScreenDC := GetDC( 0 );
    try
      BitBlt( Canvas.Handle, 0, 0, Width, Height, ScreenDC,
        Left, Top, SRCCOPY );
    finally
      ReleaseDC( 0, ScreenDC );
    end;
  end;
end;

(*
{ Use this to capture the entire screen... }
procedure CaptureScreen;
begin
  with Screen do
    CaptureScreenRect( Rect( 0, 0, Width, Height ), Form1.Image1.Picture.Bitmap);
end;
*)

function DiskTypeStr(Drive: Char): ShortString;
var
  i: word;
begin
  i := GetDriveType(PChar(LowerCase(Drive+':\')));
  case i of
    { Onbekend }
    0: result := 'onbekend type';
    { bestaat niet }
    1: result := 'disk bestaat niet';
    { Floppy }
    DRIVE_REMOVABLE: result := 'floppy';
    { Harde Schijf }
    DRIVE_FIXED: result := 'harddisk';
    { Net eenheid }
     DRIVE_REMOTE: result := 'netwerk disk';
    { CD-ROM }
    DRIVE_CDROM: result := 'cd-rom';
    { Ram Drive }
    DRIVE_RAMDISK: result := 'ram-disk';
  end;
end;

function DiskInDrive(Drive: Char): Boolean;
var
  ErrorMode: word;
begin
  { maak er een hoofdletter van }
  Drive := UpCase(Drive);
  if not (Drive in ['A'..'Z']) then
    raise EConvertError.Create('Geen geldige stationsaanduiding: '+Drive+':');
  { schakel kritieke fouten uit }
  ErrorMode := SetErrorMode(SEM_FailCriticalErrors);
  try
    { drive 1 = a, 2 = b, 3 = c, etc. }
    if DiskSize(Ord(Drive) - Ord('A') + 1) = -1 then
      Result := False
    else
      Result := True;
  finally
    { restore old error mode }
    SetErrorMode(ErrorMode);
  end;
end;


function ExecuteFileEx(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  AHandle := ShellExecute(Application.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
  if AHandle <= 32 then Result := False else Result := True;
end;


function ExecuteFile(const FileName: string): Boolean;
var
  zFileName, zParams, zDir: array[0..79] of Char;
  AHandle: THandle;
begin
  AHandle := ShellExecute(Application.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, ''),
    StrPCopy(zDir, ''), SW_SHOWNORMAL);
  if AHandle <= 32 then Result := False else Result := True;
end;

end.