unit uparser;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, FileCtrl, Misc, NlDlg32;

type
  TForm1 = class(TForm)
    ParseRtfBtn: TButton;
    RichEdit1: TRichEdit;
    DriveComboBox1: TDriveComboBox;
    DirectoryListBox1: TDirectoryListBox;
    FileListBox1: TFileListBox;
    CloseBtn: TButton;
    FilterComboBox1: TFilterComboBox;
    ParseHtmlBtn: TButton;
    procedure ParseRtfBtnClick(Sender: TObject);
    procedure ParseHtmlBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure MaakPascalLijst;
    function IsGereserveerd(W: String): Boolean;
    function IsSpace(C: Char): Boolean;
    procedure FormCreate(Sender: TObject);
    function ParseString(const S: String): String;
    procedure DirectoryListBox1Change(Sender: TObject);
    function BoldOn: String;
    function BoldOff: String;
    function TextStart: String;
    function TextStop: string;
    function CommOn: String;
    function CommOff: String;
    function NewLine: String;
    procedure Parse;
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    PascalGereserveerd: Array[0..86] of ShortString;
    IsLineComment: Boolean;
    IsBlockComment1, IsBlockComment2: Boolean;
    IsString: Boolean;
    DoRTF: Boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}




const CrLf = #13#10;
      BoldOnRtf = '\plain\f0\fs18\cf0\b ';
      BoldOffRtf = '\plain\f0\fs18\cf0 ';
      CommOnRtf = '\plain\f0\fs18\cf1\i ';
      CommOffRtf = '\plain\f0\fs18\cf0 ';
      RTFStart = '{\rtf1\ansi\ansicpg1252\deff0\deftab720'+
                 //font tabel met 1 font
                 '{\fonttbl'+
                 '{\f0\fmodern Courier New;}'+
                 '}'+
                 //kleur0 = zwart; kleur1 = blauw
                 '{\colortbl\red0\green0\blue0;\red0\green0\blue255;}'+
                 '\deflang1043\pard\plain\f0\fs18\cf0';
      RTFStop = '\par }';

      BoldOnHtml = '<b>';
      BoldOffHtml = '</b>';
      CommOnHtml = '<span class="comment">';
      CommOffHtml = '</span>';
      HTMLStart = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"'+ CrLf +
                  '"http://www.w3.org/TR/html4/strict.dtd">'+ CrLf +
                  '<html>' + CrLf +
                  '<head>' + CrLf +
                  '<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1">'+ CrLf +
                  '<meta name="generator" content="PasParser by Flying Sheep Inc.">'+ CrLf +
                  CrLf + CrLf +
                  '<style type="text/css">' + CrLf +
                  'span.comment{' + CrLf +
                  '  color: blue;' + CrLf +
                  '  font-style: italic;}' + CrLf + CrLf +
                  'code{'+ CrLf +
                  '  font-family: monospace;'+ CrLf +
                  '  white-space: pre;}'+ CrlF +
                  '</style>' + CrLf + CrLf +
                  '</HEAD>'+ CrLf +
                  '<BODY>'+ CrLf + CrLf +
                  '<p>'+ CrLf + CrLf +
                  '<pre><code>' + CrLf + CrLf;
      HTMLStop = CrLf + CrLf + '<code></pre>' + CrLf +
                 '</p>'+ CrLf + CrLf +
                 '</body>' + CrLf +
                 '</html>';

      //Mogelijke scheidingstekens tussen twee woorden
      Separators = [#32,'{','}','(',')','/','\','[',']','.',';',#9,#13,#10,#26];





procedure TForm1.FormCreate(Sender: TObject);
begin
  MaakPascalLijst;
  CloseBtn.Top := ParseRtfBtn.Top;
  CloseBtn.Left := ParseRtfBtn.Left;
  CloseBtn.Width := ParseRtfBtn.Width;
  CloseBtn.Visible := False;
  RichEdit1.Width := 529;
  RichEdit1.Height := 377;
  RichEdit1.Top := DirectoryListbox1.Top;
  RichEdit1.Left := DirectoryListbox1.Left;
  RichEdit1.Visible := false;
  DirectoryListBox1Change(Self);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  DirectoryListBox1.Width := (ClientWidth - (2*DirectoryListBox1.Left)-15) div 2;
  FileListBox1.Width := DirectoryListBox1.Width;
  FileListBox1.Left := DirectoryListBox1.Width + DirectoryListBox1.Left + 15;
  DriveComboBox1.Width := DirectoryListBox1.Width;
  FilterComboBox1.Width := FileListBox1.Width;
  FilterComboBox1.Left := FileListBox1.Left;
  RichEdit1.Width :=  DirectoryListBox1.Width + FileListBox1.Width + 15;
  CloseBtn.Width := RichEdit1.Width;
  ParseRTFBtn.Width := RichEdit1.Width;
  ParseHTMLBtn.Width := RichEdit1.Width;

  ParseHtmlBtn.Top := ClientHeight - ParseHtmlBtn.Height - 8;
  ParseRtfBtn.Top := ParseHtmlBtn.Top - ParseRtfBtn.Height - 5;
  CloseBtn.Top := ParseRtfBtn.Top;
  DriveComboBox1.Top := CloseBtn.Top - DriveComboBox1.Height - 8;
  FilterComboBox1.Top := DriveComboBox1.Top;
  DirectoryListBox1.Height := DriveComboBox1.Top - DirectoryListBox1.Top - 8;
  FileListBox1.Height := DirectoryListBox1.Height;
  Richedit1.Height := CloseBtn.Top - Richedit1.Top - 8;
end;


function TForm1.BoldOn: String;
begin
  if DoRTF then Result := BoldOnRTF
  else Result := BoldOnHTML;
end;

function TForm1.BoldOff: String;
begin
  if DoRTF then Result := BoldOffRTF
  else Result := BoldOffHTML;
end;

function TForm1.TextStart: String;
begin
  if DoRTF then Result := RTFStart
  else Result := HTMLStart;
end;

function TForm1.TextStop: string;
begin
  if DoRTF then Result := RTFStop
  else Result := HTMLStop;
end;

function TForm1.CommOn: String;
begin
  if DoRTF then Result := CommOnRTF
  else Result := CommOnHTML;
end;

function TForm1.CommOff: String;
begin
  if DoRTF then Result := CommOffRTF
  else Result := CommOffHTML;
end;

function TForm1.NewLine: String;
begin
  if DoRTF then result := ' \par'
  else Result := ''; //Geen <br> i.v.m. <code>-Tag
end;



procedure TForm1.Parse;
var SF, DF: TextFile;
    Sn, Dn: String;
    S: String;
    i: integer;
begin
  for i := 0 to FileListBox1.Items.Count - 1 do if filelistbox1.selected[i] then
  begin
    Sn := FileListBox1.Items[i];
    if IsASCIIFile(Sn) then
    begin
      if DoRTF then Dn := Sn + '.rtf' //ChangeFileExt(Sn,'.rtf')
      else Dn := Sn + '.htm'; //ChangeFileExt(Sn,'.htm');
      AssignFile(SF,Sn);
      AssignFile(DF,Dn);
      Reset(SF);
      Rewrite(DF);
      IsString := False;
      Write(DF,TextStart);
      while not Eof(SF) do
      begin
        Readln(SF,S);
        Writeln(DF,ParseString(S)+ NewLine);
      end;
      Writeln(DF,TextStop);
      CloseFile(DF);
      CloseFile(SF);
    end
    else MessageDlg('"'+Sn+'"'^m'is geen ASCII bestand.',mtError,[mbOk],0);
  end;
  if not FileExists(Dn) then
  begin
    MessageDlg('Er is geen bestand beschikbaar om als voorbeeld te tonen.',
               mtError,[mbOk],0);
    exit;
  end;
  if FileListBox1.SelCount > 1 then
  begin
    MessageDlg('Alleen het laatste bestand uit de lijst wordt als voorbeeld getoond !',
               mtInformation,[mbOk],0);
  end;
  if DoRTF then
  begin
    Richedit1.Lines.LoadFromFile(Dn);
    RichEdit1.Visible := True;
    ParseRtfBtn.Visible := False;
    ParseHtmlBtn.Visible := False;
    CloseBtn.Visible := True;
    FileListBox1.Visible := False;
    DirectoryListBox1.Visible := False;
    DriveComboBox1.Visible := False;
    FilterComboBox1.Visible := False;
  end
  else
  begin
    ExecuteFile(Dn);
  end;
end;

procedure TForm1.ParseRtfBtnClick(Sender: TObject);
begin
  DoRTF := True;
  Parse;
end;

procedure TForm1.ParseHtmlBtnClick(Sender: TObject);
begin
  DoRTF := False;
  Parse;
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
  Richedit1.Visible := False;
  ParseRtfBtn.Visible := True;
  ParseHtmlBtn.Visible := True;
  CloseBtn.Visible := False;
  FileListBox1.Visible := True;
  DirectoryListBox1.Visible := True;
  DriveComboBox1.Visible := True;
  FilterComboBox1.Visible := True
end;


procedure TForm1.DirectoryListBox1Change(Sender: TObject);
begin
  FileListBox1.ItemIndex := 0;
  if FileListBox1.Items.Count > 0 then FileListBox1.Selected[0] := true;
  ParseRtfBtn.Enabled := (FileListBox1.SelCount > 0);
  ParseHtmlBtn.Enabled := ParseRtfBtn.Enabled;
end;

function TForm1.ParseString(const S: String): String;
var i: integer;
    SS: String;
    W: String;
begin
  IsLineComment := False;
  //per definitie is het begin van een regel het eind van de vorige
  //en dus van een een vorige LineComment
  IsString := False;
  //Strings kunnen niet over meerdere regels verspreid staan
  //uitzondering is een string in een commentaar tussen {} of (* *) paren
  //deze uitzondering wordt helaas niet goed afgehandeld
  //zodat als in zo'n string een } op *) volgt
  //deze routine aanneemt dat het commentaar is afgesloten ...
  SS := '';
  W := '';
  for i := 1 to length(S) do
  begin
    if (S[i] = '''') then IsString := not IsString;
    //een string is ook een string als hij in een commentaar staat
    //in dat geval is "}" of "*)" geen afsluiting van een commentaarblok!
    if ((not IsString and not IsLineComment) and (S[i] = '{')) or
    ((not IsString) and not (IsBlockComment1 or IsBlockComment2) and not IsLineComment and (S[i] = '(') and (Length(S) > i) and (S[i+1] = '*')) then
    begin//commentaar begonnen met "{" of (*
      if (S[i]='{') then IsBlockComment1 := True else IsBlockComment2 := true;
      if IsGereserveerd(W) then W := BoldOn + W + BoldOff;
      SS := SS + W + CommOn;
      W := '';
    end;
    if (not IsString) and (not (IsBlockComment1 or IsBlockComment2)) and (not IsLineComment) and (S[i] = '/') and (Length(S) > i) and (S[i+1] = '/') then
    begin//commentaar begonnen met "//"
      IsLineComment := True;
      if Isgereserveerd(W) then W := BoldOn + W + BoldOff;
      SS := SS + W + CommOn;
      W := '';
    end;
    //"escape" tekens
    if (S[i] in ['{','}','\']) and DoRTF then W := W + '\';
    if (S[i] in ['<','>','&']) and not DoRTF then
    begin
      if S[i] = '<' then W := W + '&lt;' else
      if S[i] = '>' then W := W + '&gt;' else
      if S[i] = '&' then W := W + '&amp;';
    end
    else W := W + S[i];
    if IsSpace(S[i]) or (i = Length(S)) then
    begin//einde van een woord
      if not ((IsBlockComment1 or IsBlockComment2) or IsLineComment or IsString) and IsGereserveerd(W) then
      begin//maak gereserveerd woord vet als het niet in commentaar of string staat
        W := BoldOn + W + BoldOff;
      end;
      SS := SS + W;
      W := '';
    end;
    if (IsBlockComment1 and (not IsString) and (S[i] = '}'))  or
    (IsBlockComment2 and (not IsString) and (S[i] = ')') and (i > 1) and (S[i-1] = '*')) then
    begin//einde commentaar gestart met "{" of "(*"
      SS := SS + CommOff;
      IsBlockComment1 := False;
      IsBlockComment2 := False;
    end;
  end;
  if IsLineComment then
  begin//einde commentaar gestart met "//"
    IsLineComment := False;
    SS := SS + CommOff;
  end;
  if (SS = '') and not DoRTF then SS := '<br>';
  Result := SS;
end;


function TForm1.IsGereserveerd(W: String): Boolean;
procedure PasTrim(var S: String);
const CSet = [#32,';','.'];
begin
  While (Length(S) > 1) and (S[1] = ' ') do Delete(S,1,1);
  if S = ' ' then S := '';
  While (Length(S) > 1) and (S[Length(S)] in CSet) do Delete(S,Length(S),1);
  if (Length(S) = 1) and (S[1] in CSet) then S := '';
end;
var i: Integer;
begin
  Result := False;
  PasTrim(W);
  for i := 0 to 86 do
  begin
    if CompareText(PascalGereserveerd[i],W) = 0 then
    begin
      Result := true;
      Break;
    end;
  end;
end;



function TForm1.IsSpace(C: Char): Boolean;
begin
  Result := (C in Separators);
end;


procedure TForm1.MaakPascalLijst;
begin
  PascalGereserveerd[0] := 'and';
  PascalGereserveerd[1] := 'array';
  PascalGereserveerd[2] := 'as';
  PascalGereserveerd[3] := 'asm';
  PascalGereserveerd[4] := 'assembler';
  PascalGereserveerd[5] := 'automated';
  PascalGereserveerd[6] := 'begin';
  PascalGereserveerd[7] := 'case';
  PascalGereserveerd[8] := 'cdecl';
  PascalGereserveerd[9] := 'class';
  PascalGereserveerd[10] := 'const';
  PascalGereserveerd[11] := 'constructor';
  PascalGereserveerd[12] := 'default';
  PascalGereserveerd[13] := 'destructor';
  PascalGereserveerd[14] := 'dispinterface';
  PascalGereserveerd[15] := 'div';
  PascalGereserveerd[16] := 'do';
  PascalGereserveerd[17] := 'downto';
  PascalGereserveerd[18] := 'dynamic';
  PascalGereserveerd[19] := 'else';
  PascalGereserveerd[20] := 'end';
  PascalGereserveerd[21] := 'except';
  PascalGereserveerd[22] := 'export';
  PascalGereserveerd[23] := 'exports';
  PascalGereserveerd[24] := 'far';
  PascalGereserveerd[25] := 'file';
  PascalGereserveerd[26] := 'finalization';
  PascalGereserveerd[27] := 'finally';
  PascalGereserveerd[28] := 'for';
  PascalGereserveerd[29] := 'forward';
  PascalGereserveerd[30] := 'function';
  PascalGereserveerd[31] := 'goto';
  PascalGereserveerd[32] := 'if';
  PascalGereserveerd[33] := 'implementation';
  PascalGereserveerd[34] := 'in';
  PascalGereserveerd[35] := 'inherited';
  PascalGereserveerd[36] := 'initialization';
  PascalGereserveerd[37] := 'inline';
  PascalGereserveerd[38] := 'interface';
  PascalGereserveerd[39] := 'is';
  PascalGereserveerd[40] := 'label';
  PascalGereserveerd[41] := 'library';
  PascalGereserveerd[42] := 'message';
  PascalGereserveerd[43] := 'mod';
  PascalGereserveerd[44] := 'near';
  PascalGereserveerd[45] := 'nil';
  PascalGereserveerd[46] := 'not';
  PascalGereserveerd[47] := 'object';
  PascalGereserveerd[48] := 'of';
  PascalGereserveerd[49] := 'or';
  PascalGereserveerd[50] := 'out';
  PascalGereserveerd[51] := 'override';
  PascalGereserveerd[52] := 'packed';
  PascalGereserveerd[53] := 'private';
  PascalGereserveerd[54] := 'private';
  PascalGereserveerd[55] := 'procedure';
  PascalGereserveerd[56] := 'program';
  PascalGereserveerd[57] := 'property';
  PascalGereserveerd[58] := 'protected';
  PascalGereserveerd[59] := 'protected';
  PascalGereserveerd[60] := 'public';
  PascalGereserveerd[61] := 'public';
  PascalGereserveerd[62] := 'published';
  PascalGereserveerd[63] := 'published';
  PascalGereserveerd[64] := 'raise';
  PascalGereserveerd[65] := 'record';
  PascalGereserveerd[66] := 'repeat';
  PascalGereserveerd[67] := 'resourcestring';
  PascalGereserveerd[68] := 'set';
  PascalGereserveerd[69] := 'shl';
  PascalGereserveerd[70] := 'shr';
  PascalGereserveerd[71] := 'stdcall';
  PascalGereserveerd[72] := 'string';
  PascalGereserveerd[73] := 'stringresource';
  PascalGereserveerd[74] := 'then';
  PascalGereserveerd[75] := 'threadvar';
  PascalGereserveerd[76] := 'to';
  PascalGereserveerd[77] := 'try';
  PascalGereserveerd[78] := 'type';
  PascalGereserveerd[79] := 'unit';
  PascalGereserveerd[80] := 'until';
  PascalGereserveerd[81] := 'uses';
  PascalGereserveerd[82] := 'var';
  PascalGereserveerd[83] := 'virtual';
  PascalGereserveerd[84] := 'while';
  PascalGereserveerd[85] := 'with';
  PascalGereserveerd[86] := 'xor';
end;




end.




